@@ -1,28 +1,9 @@
-# ##### BEGIN GPL LICENSE BLOCK #####
-#
-#  This program is free software; you can redistribute it and/or
-#  modify it under the terms of the GNU General Public License
-#  as published by the Free Software Foundation; either version 2
-#  of the License, or (at your option) any later version.
-#
-#  This program is distributed in the hope that it will be useful,
-#  but WITHOUT ANY WARRANTY; without even the implied warranty of
-#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-#  GNU General Public License for more details.
-#
-#  You should have received a copy of the GNU General Public License
-#  along with this program; if not, write to the Free Software Foundation,
-#  Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
-#
-# ##### END GPL LICENSE BLOCK #####
-
-# <pep8-80 compliant>
+# SPDX-License-Identifier: GPL-2.0-or-later
 
 # Originally written by Matt Ebb
 
 import bpy
 from bpy.types import Operator
-import os
 
 from bpy.app.translations import pgettext_tip as tip_
 
@@ -61,7 +42,28 @@ class PlayRenderedAnim(Operator):
     bl_label = "Play Rendered Animation"
     bl_options = {'REGISTER'}
 
+    @staticmethod
+    def _frame_path_with_number_char(rd, ch, **kwargs):
+        # Replace the number with `ch`.
+
+        # NOTE: make an api call for this would be nice, however this isn't needed in many places.
+        file_a = rd.frame_path(frame=0, **kwargs)
+        file_b = rd.frame_path(frame=-1, **kwargs)
+        assert len(file_b) == len(file_a) + 1
+
+        for number_beg in range(len(file_a)):
+            if file_a[number_beg] != file_b[number_beg]:
+                break
+
+        for number_end in range(-1, -(len(file_a) + 1), -1):
+            if file_a[number_end] != file_b[number_end]:
+                break
+
+        number_end += len(file_a) + 1
+        return file_a[:number_beg] + (ch * (number_end - number_beg)) + file_a[number_end:]
+
     def execute(self, context):
+        import os
         import subprocess
         from shlex import quote
 
@@ -74,6 +76,12 @@ class PlayRenderedAnim(Operator):
         # file_path = bpy.path.abspath(rd.filepath)  # UNUSED
         is_movie = rd.is_movie_format
 
+        views_format = rd.image_settings.views_format
+        if rd.use_multiview and views_format == 'INDIVIDUAL':
+            view_suffix = rd.views.active.file_suffix
+        else:
+            view_suffix = ""
+
         # try and guess a command line if it doesn't exist
         if preset == 'CUSTOM':
             player_path = prefs.filepaths.animation_player
@@ -81,26 +89,12 @@ class PlayRenderedAnim(Operator):
             player_path = guess_player_path(preset)
 
         if is_movie is False and preset in {'FRAMECYCLER', 'RV', 'MPLAYER'}:
-            # replace the number with '#'
-            file_a = rd.frame_path(frame=0)
-
-            # TODO, make an api call for this
-            frame_tmp = 9
-            file_b = rd.frame_path(frame=frame_tmp)
-
-            while len(file_a) == len(file_b):
-                frame_tmp = (frame_tmp * 10) + 9
-                file_b = rd.frame_path(frame=frame_tmp)
-            file_b = rd.frame_path(frame=int(frame_tmp / 10))
-
-            file = ("".join((c if file_b[i] == c else "#")
-                            for i, c in enumerate(file_a)))
-            del file_a, file_b, frame_tmp
+            file = PlayRenderedAnim._frame_path_with_number_char(rd, "#", view=view_suffix)
             file = bpy.path.abspath(file)  # expand '//'
         else:
             path_valid = True
             # works for movies and images
-            file = rd.frame_path(frame=scene.frame_start, preview=scene.use_preview_range)
+            file = rd.frame_path(frame=scene.frame_start, preview=scene.use_preview_range, view=view_suffix)
             file = bpy.path.abspath(file)  # expand '//'
             if not os.path.exists(file):
                 err_msg = tip_("File %r not found") % file
@@ -109,7 +103,7 @@ class PlayRenderedAnim(Operator):
 
             # one last try for full range if we used preview range
             if scene.use_preview_range and not path_valid:
-                file = rd.frame_path(frame=scene.frame_start, preview=False)
+                file = rd.frame_path(frame=scene.frame_start, preview=False, view=view_suffix)
                 file = bpy.path.abspath(file)  # expand '//'
                 err_msg = tip_("File %r not found") % file
                 if not os.path.exists(file):
@@ -130,6 +124,7 @@ class PlayRenderedAnim(Operator):
                 "-s", str(frame_start),
                 "-e", str(frame_end),
                 "-j", str(scene.frame_step),
+                "-c", str(prefs.system.memory_cache_limit),
                 file,
             ]
             cmd.extend(opts)
